from load_word_table import load_word_table
from data_store import get_data


def get_latin_word_from_chuhannom(
    handle,
    input_chuhannom: str,
    normalize_output_case: bool = False
):
    result_latin_words = set()
    # word_entry_list = load_word_table()
    word_entry_list = get_data(handle)
    input_chuhannom = input_chuhannom.strip()
    for each_word_entry in word_entry_list:
        if input_chuhannom in each_word_entry[0]:
            if normalize_output_case == False:
                result_latin_words.add(tuple(each_word_entry))
            if normalize_output_case == True:
                result_latin_words.add(tuple([each_word_entry[0], each_word_entry[1].lower()]))
    return result_latin_words




from load_word_table import load_word_table
from data_store import get_data


def get_chuhannom_word_from_latin(
    handle,
    input_latin_syllables: str,
    normalize_input_case: bool = False,   # 是否在匹配前统一大小写
    case_insensitive_match: bool = True   # 匹配时是否忽略大小写
):
    result_word_entry = set()
    # word_entry_list = load_word_table()
    word_entry_list = get_data(handle)
    input_latin_syllables = input_latin_syllables.strip()
    if normalize_input_case == True:
        input_latin_syllables = input_latin_syllables.lower()
    input_latin_syllable_list = input_latin_syllables.split()

    if case_insensitive_match == False:
        for each_word_entry in word_entry_list:
            each_word_entry_check_list = []
            for each_input_latin_syllable in input_latin_syllable_list:
                if each_input_latin_syllable in each_word_entry[1]:
                    each_word_entry_check_list.append(True)
                else:
                    each_word_entry_check_list.append(False)
            if False not in each_word_entry_check_list:
                # print(each_word_entry)
                result_word_entry.add(tuple(each_word_entry))
    if case_insensitive_match == True:
        for each_word_entry in word_entry_list:
            each_word_entry_check_list = []
            for each_input_latin_syllable in input_latin_syllable_list:
                if each_input_latin_syllable.lower() in each_word_entry[1].lower():
                    each_word_entry_check_list.append(True)
                else:
                    each_word_entry_check_list.append(False)
            if False not in each_word_entry_check_list:
                # print(each_word_entry)
                result_word_entry.add(tuple(each_word_entry))
    return result_word_entry




"""
PyHanNom: Vietnamese Latin ↔ Han-Nom converter and lookup tool
"""

from .load_syllable_char_table import load_syllable_char_table
from .get_chuhannom_from_latin import (
    get_chuhannom_from_latin,
    get_chuhannom_unicode_from_latin,
)
from .get_latin_from_chuhannom import get_latin_from_chuhannom

from .load_word_table import load_word_table
from .get_chuhannom_word_from_latin import get_chuhannom_word_from_latin
from .get_latin_word_from_chuhannom import get_latin_word_from_chuhannom

__version__ = "0.1.0"

__all__ = [
    "load_syllable_char_table",
    "get_chuhannom_from_latin",
    "get_chuhannom_unicode_from_latin",
    "get_latin_from_chuhannom",
    "load_word_table",
    "get_chuhannom_word_from_latin",
    "get_latin_word_from_chuhannom",
]



import re
from data_store import store_data


def load_syllable_char_table():
    """
    解析 rawCharWordTable.txt，将需要的行合并并拆分出 Unicode 部分和前置内容。

    参数:
        file_path (str): 输入文件路径

    返回:
        list[list[str]]: 处理后的二维列表，每行格式为：
                         [原行的前几列..., Unicode部分, 前置内容]
    """
    # 读取文件并按行拆分为列表
    with open('data/rawCharWordTable.txt', 'r', encoding='utf-8') as f:
        table_rows = [line.strip('\n').split('\t') for line in f]

    processed_rows = []
    row_index = 1  # 跳过表头

    while row_index < len(table_rows):
        current_row = table_rows[row_index]

        # 如果最后一列不包含 'U+'，需要和后续行合并
        if 'U+' not in current_row[-1]:
            merge_index = row_index + 1
            while merge_index < len(table_rows):
                next_row = table_rows[merge_index]
                current_row[-1] += '\n' + next_row[0]
                if 'U+' in next_row[0]:
                    break
                merge_index += 1
            row_index = merge_index + 1
        else:
            row_index += 1

        # 拆分 Unicode 部分
        last_column = current_row[-1]
        unicode_pos = last_column.find("U+")
        if unicode_pos != -1:
            before_unicode = last_column[:unicode_pos].strip().replace('\n', ' ')
            unicode_part = last_column[unicode_pos:].strip()
            processed_row = current_row[:-1] + [unicode_part] + [before_unicode]
            processed_rows.append(processed_row)
        else:
            print("U+ NOT FOUND")
            return ["U+ NOT FOUND"]
    return store_data(processed_rows)



from data_store import store_data
from data_store import get_data
# from load_syllable_char_table_base_20250917 import load_syllable_char_table
import unicodedata
import re

def to_ascii(s):
    # NFD 分解 -> 去掉音符 -> 转回 str
    return unicodedata.normalize('NFD', s).encode('ascii', 'ignore').decode('ascii')

def is_latin_or_digit(s):
    ascii_str = to_ascii(s)
    # 如果全是数字
    if ascii_str.isdigit():
        return True
    # 如果包含至少一个英文字母
    if re.search(r'[A-Za-z]', ascii_str):
        return True
    return False

def load_word_table(handle):
    word_entry_list = []
    province_name_entry = []
    country_name_entry = []  # 用来存储处理后的行
    full_final_word_entry_list = []
    with open('data/country_name.txt', 'r', encoding='utf-8') as country_name_f:
        for line in country_name_f:
            # 去掉行尾换行符
            stripped_line = line.strip('\n')
            # 按制表符分割成列表
            parts = stripped_line.split('\t')
            each_country_entry = [parts[1].strip(), parts[0].strip()]
            # 添加到结果列表
            country_name_entry.append(each_country_entry)
    # print(country_name_entry)

    with open('data/vietnam_province_name.txt', 'r', encoding='utf-8') as province_name_f:
        for line in province_name_f:
            stripped_line = line.strip('\n')
            stripped_line_list = stripped_line.split(' ')
            # print(stripped_line_list)
            result = []
            for item in stripped_line_list:
                if is_latin_or_digit(item) and result and is_latin_or_digit(result[-1]):
                    result[-1] += ' ' + item
                else:
                    result.append(item)
            # print(result)
            for ele_num in range(len(result)):
                if ele_num % 2 == 0:
                    province_name_entry.append([result[ele_num].strip(), result[ele_num + 1].strip()])
    # print(province_name_entry)

    # char_word_table = load_syllable_char_table('rawCharWordTable.txt')
    char_word_table = get_data(handle)
    for each_syllable_char_info_line in char_word_table:
        word_list_in_each_line = each_syllable_char_info_line[2].split(' · ')
        for each_word in word_list_in_each_line:
            if len(each_word) > 0:
                word_entry = each_word.split(" ", 1)
                non_cjkv_part = word_entry[1].split('[')
                if len(non_cjkv_part) > 1:
                    non_cjkv_part = [non_cjkv_part[0].strip(), ('[' + non_cjkv_part[1]).strip()]
                cjkv_part = word_entry[0].strip()
                word_entry = [cjkv_part] + non_cjkv_part
                # print(word_entry)
                word_entry_list.append(word_entry)
    full_final_word_entry_list = word_entry_list + country_name_entry + province_name_entry
    return store_data(full_final_word_entry_list)




from load_syllable_char_table import load_syllable_char_table
from get_chuhannom_from_latin import get_chuhannom_from_latin
from get_chuhannom_from_latin import get_chuhannom_unicode_from_latin
from get_latin_from_chuhannom import get_latin_from_chuhannom

from load_word_table import load_word_table
from get_chuhannom_word_from_latin import get_chuhannom_word_from_latin
from get_latin_word_from_chuhannom import get_latin_word_from_chuhannom


hannom_syllable_char_table = load_syllable_char_table()
got_hannom_char = get_chuhannom_from_latin(hannom_syllable_char_table, 'bàn')
got_hannom_unicode = get_chuhannom_unicode_from_latin(hannom_syllable_char_table, 'buộc')
got_latin_syllable = get_latin_from_chuhannom(hannom_syllable_char_table, '心')

hannom_word_table = load_word_table(hannom_syllable_char_table)
got_hannom_word = get_chuhannom_word_from_latin(hannom_word_table, 'hưng hửng')
got_latin_word = get_latin_word_from_chuhannom(hannom_word_table, '汴𠲅')


print(hannom_syllable_char_table)
print(got_hannom_char)
print(got_hannom_unicode)
print(got_latin_syllable)

print(hannom_word_table)
print(got_hannom_word)
print(got_latin_word)


# 内部缓存（用户无法直接访问）
_cache = {}

class DataHandle:
    def __init__(self, key):
        self._key = key
    def __repr__(self):
        return f"<DataHandle id={self._key}>"

def store_data(data):
    """存储数据并返回句柄"""
    key = id(data)
    _cache[key] = data
    return DataHandle(key)

def get_data(handle):
    """根据句柄取数据（仅库内部使用）"""
    return _cache.get(handle._key)



from load_syllable_char_table import load_syllable_char_table
from data_store import get_data


def get_chuhannom_from_latin(
    handle,
    input_latin_syllable: str,
    normalize_input_case: bool = True,   # 是否在匹配前统一大小写
    case_insensitive_match: bool = True   # 匹配时是否忽略大小写
):
    result_chars = []
    char_word_table = get_data(handle)
    # char_word_table = load_syllable_char_table('rawCharWordTable.txt')
    input_latin_syllable = input_latin_syllable.strip()
    if normalize_input_case == True:
        input_latin_syllable = input_latin_syllable.lower()
    for each_syllable_char_info_line in char_word_table:
        if case_insensitive_match == True:
            if each_syllable_char_info_line[0].lower() == input_latin_syllable.lower():
                result_chars.append(each_syllable_char_info_line[1])
        else:
            if each_syllable_char_info_line[0] == input_latin_syllable:
                result_chars.append(each_syllable_char_info_line[1])
    return result_chars

def get_chuhannom_unicode_from_latin(
    handle,
    input_latin_syllable: str,
    normalize_input_case: bool = True,   # 是否在匹配前统一大小写
    case_insensitive_match: bool = True   # 匹配时是否忽略大小写
):
    result_chars_unicode = []
    char_word_table = get_data(handle)
    # char_word_table = load_syllable_char_table('rawCharWordTable.txt')
    if normalize_input_case == True:
        input_latin_syllable = input_latin_syllable.lower()
    for each_syllable_char_info_line in char_word_table:
        if case_insensitive_match == True:
            if each_syllable_char_info_line[0].lower() == input_latin_syllable.lower():
                result_chars_unicode.append(each_syllable_char_info_line[3])
        else:
            if each_syllable_char_info_line[0] == input_latin_syllable:
                result_chars_unicode.append(each_syllable_char_info_line[3])
    return result_chars_unicode




from load_syllable_char_table import load_syllable_char_table
from data_store import get_data

def get_latin_from_chuhannom(handle, input_chuhannom: str, normalize_output_case: bool = False):
    result_latin_syllables = []
    # char_word_table = load_syllable_char_table('rawCharWordTable.txt')
    char_word_table = get_data(handle)
    input_chuhannom = input_chuhannom.strip()
    for each_syllable_char_info_line in char_word_table:
        if input_chuhannom in each_syllable_char_info_line[1]:
            if normalize_output_case == False:
                result_latin_syllables.append(each_syllable_char_info_line[0])
            else:
                result_latin_syllables.append(each_syllable_char_info_line[0].lower())
    return result_latin_syllables
